/*
 * BitMapsInputStream.java
 *
 * Created on 25.12. 2003, 22:54
 */

package saiko.compression;

import java.io.*;

/**
 * Class to provide simple RLE algorithm decompression input stream
 *
 * @author  Dusan Saiko
 * @see saiko.compression.RLEOutputStream
 */
public class BitMapsInputStream extends FilterInputStream {
    
   
    private int theChar;
    
    /** 
     * Constructor of new instance of RLEInputStream
     *
     * @param aOutput (OutputStream) - output stream to write write the result into
     * @exception  IOException if an I/O error occurs or if no valid header found
    */
    public BitMapsInputStream(InputStream aInput) throws IOException
    {
        super(aInput);
        
        byte header[]=new byte[BitMapsOutputStream.header.length()];
        in.read(header);
        if(!BitMapsOutputStream.header.equals(new String(header)))
            throw new IOException("Invalid data format - not BM data stream");
        theChar=in.read();
    }

   
    int currentMap=0;
    int currentMapPos=0;
    
    /**
     * Reads the next byte from this compressed output stream. 
     *
     * @exception  IOException  if an I/O error occurs.
     */
    public synchronized int read() throws IOException {
        // if we do not have the map set, read next byte
        if(currentMapPos==0) {
            int c=in.read();
            //if this is end or if the byte does not have first bit set,
            //return in
            if((c==-1) || ((c & 0x80)==0) ) return c;
            //we have read the bit map
            currentMap=c;
            //set the current position on the second bit
            currentMapPos=(1<<6);
        }
        // we deal with the map
        int res;
        if((currentMap & currentMapPos)!=0) {
            //map indicates to read the next char from input
            res=in.read();
        } else {
            //map indicates to return the 'background' character
            res=theChar;
        }
        //shift the map position
        currentMapPos=currentMapPos>>1;
        return res;
    } //read
    
  /**
     * reads the array of data
     * @param b - the buffer into which the data is read.
     * @param off - the start offset of the data.
     * @param len - the maximum number of bytes read. 
     * @return the total number of bytes read into the buffer, 
        or -1 if there is no more data because the end of the stream has been reached. 
     */
    public synchronized int read(byte b[], int off, int len)
    throws IOException
    {
        if ((off | len | (off + len) | (b.length - (off + len))) < 0) {
	    throw new IndexOutOfBoundsException();
	} else if (len == 0) {
	    return 0;
	}

	int n = 0;
	while (n < len) {
	    int n1 = read();
	    if (n1 < 0) break;
            b[off+n]=(byte)n1;
	    n ++;
	}
	return n==0 ? -1 : n;
    }     
}
